/*
 * Change History
 * 2011.03.28 - Fixed bug with negative and positive longitudes
 */

package getpde;

import java.io.File;

public class Main {

    public static Configuration config; //Configuration object

    public static void main(String[] args) {
        /* Set system time */
        System.setProperty("user.timezone", "Etc/Universal");

        //Search for configuration file
        File configfile = new File("getPDE.xml");
        if (args.length>0) {
            //There is an input argument, maybe its the configuration file
            File temp = new File(args[0]);
            if (temp.exists()) {
                System.out.println("Using configuration file: " + temp.getName());
                configfile = temp;
            } else {
                System.out.println("Configuration file: " + temp.getName() + " not found.");
                System.exit(-1);
            }
        } else {
            System.out.println("Searching for default configuration file, getPDE.xml");
            if (!configfile.exists()) {
                configfile = new File(System.getenv("SEISAN_TOP") + File.separator + "DAT" + File.separator + "getPDE.xml");
            }
        }
        if (!configfile.exists()) {
            System.out.println("Configuration file not found.");
            System.exit(-1);
        }

        try {
            config = Configuration.loadConfig(configfile);
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }

        //Start console
        new Console(config).start();

        //Start acquisition thread
        new DataAcquisition(config).start();
    }

}
